@@ -293,8 +293,16 @@ retrieval_done (SoupMessage *msg, ECalBackendHttp *cbhttp)
 	while (subcomp) {
 		ECalComponent *comp;
 		icalcomponent_kind subcomp_kind;
+		icalproperty *prop = NULL;
 
 		subcomp_kind = icalcomponent_isa (subcomp);
+		prop = icalcomponent_get_first_property (subcomp, ICAL_UID_PROPERTY);
+		if (!prop) {
+			g_warning (" The component does not have the  mandatory property UID \n");
+			subcomp = icalcomponent_get_next_component (icalcomp, kind);
+			continue;
+		}
+
 		if (subcomp_kind == kind) {
 			comp = e_cal_component_new ();
 			if (e_cal_component_set_icalcomponent (comp, subcomp)) {
@@ -960,6 +968,8 @@ e_cal_backend_http_init (ECalBackendHttp *cbhttp, ECalBackendHttpClass *class)
 	priv->uri = NULL;
 	priv->reload_timeout_id = 0;
 	priv->opened = FALSE;
+
+	e_cal_backend_sync_set_lock (E_CAL_BACKEND_SYNC (cbhttp), TRUE);
 }
 
 /* Class initialization function for the file backend */
