@@ -663,7 +663,7 @@ get_folder_info (CamelStore *store, const char *top, guint32 flags, CamelExcepti
 }
 
 static void
-add_special_info (CamelStore *store, CamelFolderInfo *info, const char *name, const char *translated, gboolean unread_count)
+add_special_info (CamelStore *store, CamelFolderInfo *info, const char *name, const char *translated, gboolean unread_count, guint32 flags)
 {
 	CamelFolderInfo *fi, *vinfo, *parent;
 	char *uri, *path;
@@ -673,7 +673,7 @@ add_special_info (CamelStore *store, CamelFolderInfo *info, const char *name, co
 	
 	parent = NULL;
 	for (fi = info; fi; fi = fi->next) {
-		if (!strcmp (fi->name, name))
+		if (!strcmp (fi->full_name, name))
 			break;
 		parent = fi;
 	}
@@ -711,7 +711,7 @@ add_special_info (CamelStore *store, CamelFolderInfo *info, const char *name, co
 	}
 	
 	/* Fill in the new fields */
-	vinfo->flags |= CAMEL_FOLDER_VIRTUAL|CAMEL_FOLDER_SYSTEM|CAMEL_FOLDER_VTRASH;
+	vinfo->flags |= flags;
 	vinfo->full_name = g_strdup (name);
 	vinfo->name = g_strdup (translated);
 	vinfo->uri = uri;
@@ -775,9 +775,11 @@ camel_store_get_folder_info(CamelStore *store, const char *top, guint32 flags, C
 	
 	if (info && (top == NULL || *top == '\0') && (flags & CAMEL_STORE_FOLDER_INFO_NO_VIRTUAL) == 0) {
 		if (info->uri && (store->flags & CAMEL_STORE_VTRASH))
-			add_special_info (store, info, CAMEL_VTRASH_NAME, _("Trash"), FALSE);
+			/* the name of the Trash folder, used for deleted messages */
+			add_special_info (store, info, CAMEL_VTRASH_NAME, _("Trash"), FALSE, CAMEL_FOLDER_VIRTUAL|CAMEL_FOLDER_SYSTEM|CAMEL_FOLDER_VTRASH|CAMEL_FOLDER_TYPE_TRASH);
 		if (info->uri && (store->flags & CAMEL_STORE_VJUNK))
-			add_special_info (store, info, CAMEL_VJUNK_NAME, _("Junk"), TRUE);
+			/* the name of the Junk folder, used for spam messages */
+			add_special_info (store, info, CAMEL_VJUNK_NAME, _("Junk"), TRUE, CAMEL_FOLDER_VIRTUAL|CAMEL_FOLDER_SYSTEM|CAMEL_FOLDER_VTRASH|CAMEL_FOLDER_TYPE_JUNK);
 	}
 
 	if (camel_debug_start("store:folder_info")) {
@@ -943,6 +945,7 @@ camel_folder_info_build (GPtrArray *folders, const char *namespace,
 			} else {
 				/* we are missing a folder in the heirarchy so
 				   create a fake folder node */
+				const char *path;
 				CamelURL *url;
 				char *sep;
 
@@ -955,17 +958,21 @@ camel_folder_info_build (GPtrArray *folders, const char *namespace,
 						pfi->name = g_strdup (pname);
 				} else
 					pfi->name = g_strdup (pname);
-
-				/* FIXME: url's with fragments should have the fragment truncated, not path */
+				
 				url = camel_url_new (fi->uri, NULL);
-				sep = strrchr (url->path, separator);
+				if (url->fragment)
+					path = url->fragment;
+				else
+					path = url->path + 1;
+				
+				sep = strrchr (path, separator);
 				if (sep)
 					*sep = '\0';
 				else
 					d(g_warning ("huh, no \"%c\" in \"%s\"?", separator, fi->url));
 				
-				pfi->full_name = g_strdup(url->path+1);
-
+				pfi->full_name = g_strdup (path);
+				
 				/* since this is a "fake" folder node, it is not selectable */
 				camel_url_set_param (url, "noselect", "yes");
 				pfi->uri = camel_url_to_string (url, 0);
@@ -1192,9 +1199,15 @@ camel_store_folder_uri_equal (CamelStore *store, const char *uri0, const char *u
 			name0 = url0->fragment;
 			name1 = url1->fragment;
 		} else {
-			name0 = url0->path[0] == '/' ? url0->path + 1 : url0->path;
-			name1 = url1->path[0] == '/' ? url1->path + 1 : url1->path;
+			name0 = url0->path && url0->path[0] == '/' ? url0->path + 1 : url0->path;
+			name1 = url1->path && url1->path[0] == '/' ? url1->path + 1 : url1->path;
 		}
+
+		if (name0 == NULL)
+			g_warning("URI is badly formed, missing folder name: %s", uri0);
+
+		if (name1 == NULL)
+			g_warning("URI is badly formed, missing folder name: %s", uri1);
 		
 		equal = name0 && name1 && CS_CLASS (store)->compare_folder_name (name0, name1);
 	}
