@@ -1015,7 +1015,10 @@ imap_build_folder_info(CamelImapStore *imap_store, const char *folder_name)
 		name = fi->full_name;
 	else
 		name++;
-	fi->name = g_strdup (name);
+	if (!g_ascii_strcasecmp (fi->full_name, "INBOX"))
+		fi->name = g_strdup (_("Inbox"));
+	else
+		fi->name = g_strdup (name);
 	
 	return fi;
 }
@@ -2418,10 +2421,13 @@ parse_list_response_as_folder_info (CamelImapStore *imap_store,
 	flags = (flags & ~CAMEL_FOLDER_SUBSCRIBED) | (si->info.flags & CAMEL_STORE_FOLDER_INFO_SUBSCRIBED);
 
 	fi = g_new0 (CamelFolderInfo, 1);
-	fi->name = g_strdup(camel_store_info_name(imap_store->summary, si));
 	fi->full_name = g_strdup(camel_store_info_path(imap_store->summary, si));
-	if (!g_ascii_strcasecmp(fi->full_name, "inbox"))
+	if (!g_ascii_strcasecmp(fi->full_name, "inbox")) {
 		flags |= CAMEL_FOLDER_SYSTEM|CAMEL_FOLDER_TYPE_INBOX;
+		fi->name = g_strdup (_("Inbox"));
+	} else 
+		fi->name = g_strdup(camel_store_info_name(imap_store->summary, si));
+	
 	/* HACK: some servers report noinferiors for all folders (uw-imapd)
 	   We just translate this into nochildren, and let the imap layer enforce
 	   it.  See create folder */
