@@ -26,7 +26,7 @@
 #endif
 #include <string.h>
 #include <ctype.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n-lib.h>
 #include <libsoup/soup-session-sync.h>
 #include <libsoup/soup-soap-message.h>
 #include <libsoup/soup-misc.h>
@@ -47,6 +47,11 @@ struct _EGwConnectionPrivate {
 	char *user_name;
 	char *user_email;
 	char *user_uuid;
+	char *version;
+	char *server_time ;
+	GHashTable *categories_by_name;
+	GHashTable *categories_by_id;
+	GList *book_list;
 	EGwSendOptions *opts;
 	GMutex *reauth_mutex;
 };
@@ -73,7 +78,7 @@ reauthenticate (EGwConnection *cnc)
         if (!response) {
                 g_object_unref (msg);
 		g_mutex_unlock (priv->reauth_mutex);
-                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+                return E_GW_CONNECTION_STATUS_NO_RESPONSE;
         }
         status = e_gw_connection_parse_response_status (response);
 	g_object_unref (response);
@@ -154,6 +159,8 @@ e_gw_connection_get_error_message (EGwConnectionStatus status)
 		return _("Invalid object");
 	case E_GW_CONNECTION_STATUS_INVALID_RESPONSE :
 		return _("Invalid response from server");
+	case E_GW_CONNECTION_STATUS_NO_RESPONSE:
+		return _("No response from the server");
 	case E_GW_CONNECTION_STATUS_OBJECT_NOT_FOUND :
 		return _("Object not found");
 	case E_GW_CONNECTION_STATUS_UNKNOWN_USER :
@@ -187,7 +194,7 @@ logout (EGwConnection *cnc)
 	response = e_gw_connection_send_message (cnc, msg);
 	if (!response) {
 		g_object_unref (msg);
-		return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+		return E_GW_CONNECTION_STATUS_NO_RESPONSE;
 	}
 
 	status = e_gw_connection_parse_response_status (response);
@@ -271,11 +278,37 @@ e_gw_connection_dispose (GObject *object)
 			g_mutex_free (priv->reauth_mutex);
 			priv->reauth_mutex = NULL;
 		}
-
+	
+		if (priv->categories_by_id) {
+			g_hash_table_destroy (priv->categories_by_id);
+			priv->categories_by_id = NULL;
+		}	
+		
+		if (priv->categories_by_name) {
+			g_hash_table_destroy (priv->categories_by_name);
+			priv->categories_by_name = NULL;
+		}
+	
+		if (priv->book_list) {
+			g_list_foreach (priv->book_list, (GFunc) g_object_unref, NULL);
+			g_list_free (priv->book_list);
+			priv->book_list = NULL;
+		}
+		
 		if (priv->opts) {
 			g_object_unref (priv->opts);
 			priv->opts = NULL;
 		}
+		
+		if (priv->version) {
+			g_free (priv->version) ;
+			priv->opts = NULL ;
+		}
+
+		if (priv->server_time) {
+			g_free (priv->server_time) ;
+			priv->server_time = NULL ;
+		}
 	}
 
 	if (parent_class->dispose)
@@ -323,6 +356,9 @@ e_gw_connection_init (EGwConnection *cnc, EGwConnectionClass *klass)
 	/* create the SoupSession for this connection */
 	priv->soup_session = soup_session_sync_new ();
 	priv->reauth_mutex = g_mutex_new ();
+	priv->categories_by_id = NULL;
+	priv->categories_by_name = NULL;
+	priv->book_list = NULL;
 	priv->opts = NULL;
 }
 
@@ -425,7 +461,7 @@ e_gw_connection_new (const char *uri, const char *username, const char *password
 		port = soup_soap_parameter_get_string_value (subparam);
 		if (host && port) {
 			tokens = g_strsplit (uri, "://", 2);
-			redirected_uri = g_strconcat (tokens[0], "://", host, ":", port, "/soap");
+			redirected_uri = g_strconcat (tokens[0], "://", host, ":", port, "/soap", NULL);
 			g_object_unref (msg);
 			g_object_unref (response);
 			msg = form_login_request (redirected_uri, username, password);
@@ -454,29 +490,42 @@ e_gw_connection_new (const char *uri, const char *username, const char *password
 
 	/* retrieve user information */
 	param = soup_soap_response_get_first_parameter_by_name (response, "userinfo");
+	
 	if (param) {
 		SoupSoapParameter *subparam;
-		const char *param_value;
+		char *param_value;
 
 		subparam = soup_soap_parameter_get_first_child_by_name (param, "email");
 		if (subparam) {
 			param_value = soup_soap_parameter_get_string_value (subparam);
-			cnc->priv->user_email  = g_strdup (param_value);
+			cnc->priv->user_email  = param_value;
 		}
 
 		subparam = soup_soap_parameter_get_first_child_by_name (param, "name");
 		if (subparam) {
 			param_value = soup_soap_parameter_get_string_value (subparam);
-			cnc->priv->user_name = g_strdup (param_value);
+			cnc->priv->user_name = param_value;
 		}
 
 		subparam = soup_soap_parameter_get_first_child_by_name (param, "uuid");
 		if (subparam) {
 			param_value = soup_soap_parameter_get_string_value (subparam);
-			cnc->priv->user_uuid = g_strdup (param_value);
+			cnc->priv->user_uuid = param_value;
 		}
 	}
 
+	param = soup_soap_response_get_first_parameter_by_name (response, "gwVersion");
+	if (param) {
+		char *param_value;
+		param_value = soup_soap_parameter_get_string_value (param);
+		cnc->priv->version = param_value;
+	} else
+	       	cnc->priv->version = NULL;	
+
+	param = soup_soap_response_get_first_parameter_by_name (response, "serverUTCTime");
+	if (param) 
+		cnc->priv->server_time = soup_soap_parameter_get_string_value (param);
+
 	/* add the connection to the loaded_connections hash table */
 	hash_key = g_strdup_printf ("%s:%s@%s",
 				    cnc->priv->username ? cnc->priv->username : "",
@@ -545,7 +594,7 @@ e_gw_connection_get_container_list (EGwConnection *cnc, const char *top, GList *
 	response = e_gw_connection_send_message (cnc, msg);
         if (!response) {
                 g_object_unref (msg);
-                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+                return E_GW_CONNECTION_STATUS_NO_RESPONSE;
         }
 
         status = e_gw_connection_parse_response_status (response);
@@ -560,10 +609,7 @@ e_gw_connection_get_container_list (EGwConnection *cnc, const char *top, GList *
 
 	/* if status is OK - parse result. return the list */	
 	param = soup_soap_response_get_first_parameter_by_name (response, "folders");	
-        if (!param) {
-                g_object_unref (response);
-                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
-        } else {
+        if (param) {
 		SoupSoapParameter *subparam;
 		for (subparam = soup_soap_parameter_get_first_child_by_name (param, "folder");
 		     subparam != NULL;
@@ -650,7 +696,7 @@ e_gw_connection_get_items (EGwConnection *cnc, const char *container, const char
         response = e_gw_connection_send_message (cnc, msg);
         if (!response) {
                 g_object_unref (msg);
-                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+                return E_GW_CONNECTION_STATUS_NO_RESPONSE;
         }
 
         status = e_gw_connection_parse_response_status (response);
@@ -721,7 +767,7 @@ e_gw_connection_get_items_from_ids (EGwConnection *cnc, const char *container, c
         response = e_gw_connection_send_message (cnc, msg);
         if (!response) {
                 g_object_unref (msg);
-                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+                return E_GW_CONNECTION_STATUS_NO_RESPONSE;
         }
 
         status = e_gw_connection_parse_response_status (response);
@@ -787,7 +833,7 @@ e_gw_connection_get_deltas ( EGwConnection *cnc, GSList **adds, GSList **deletes
          if (!response) { 
                  g_object_unref (msg); 
 		 // g_object_unref (cnc); 
-                 return E_GW_CONNECTION_STATUS_INVALID_RESPONSE; 
+                 return E_GW_CONNECTION_STATUS_NO_RESPONSE; 
          } 
 
          status = e_gw_connection_parse_response_status (response); 
@@ -923,7 +969,7 @@ e_gw_connection_send_item (EGwConnection *cnc, EGwItem *item, GSList **id_list)
 	response = e_gw_connection_send_message (cnc, msg);
 	if (!response) {
 		g_object_unref (msg);
-		return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+		return E_GW_CONNECTION_STATUS_NO_RESPONSE;
 	}
 
 	status = e_gw_connection_parse_response_status (response);
@@ -976,7 +1022,7 @@ e_gw_connection_create_item (EGwConnection *cnc, EGwItem *item, char** id)
 	response = e_gw_connection_send_message (cnc, msg);
 	if (!response) {
 		g_object_unref (msg);
-		return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+		return E_GW_CONNECTION_STATUS_NO_RESPONSE;
 	}
 
 	status = e_gw_connection_parse_response_status (response);
@@ -1023,7 +1069,7 @@ e_gw_connection_modify_item (EGwConnection *cnc, const char *id , EGwItem *item)
 	response = e_gw_connection_send_message (cnc, msg);
 	if (!response) {
 		g_object_unref (msg);
-		return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+		return E_GW_CONNECTION_STATUS_NO_RESPONSE;
 	}
 
 	status = e_gw_connection_parse_response_status (response);
@@ -1065,7 +1111,7 @@ e_gw_connection_get_item (EGwConnection *cnc, const char *container, const char
         response = e_gw_connection_send_message (cnc, msg);
         if (!response) {
                 g_object_unref (msg);
-                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+                return E_GW_CONNECTION_STATUS_NO_RESPONSE;
         }
 
         status = e_gw_connection_parse_response_status (response);
@@ -1121,7 +1167,7 @@ e_gw_connection_remove_item (EGwConnection *cnc, const char *container, const ch
 	response = e_gw_connection_send_message (cnc, msg);
 	if (!response) {
 		g_object_unref (msg);
-		return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+		return E_GW_CONNECTION_STATUS_NO_RESPONSE;
 	}
 
 	status = e_gw_connection_parse_response_status (response);
@@ -1158,7 +1204,7 @@ e_gw_connection_remove_items (EGwConnection *cnc, const char *container, GList *
 	response = e_gw_connection_send_message (cnc, msg);
 	if (!response) {
 		g_object_unref (msg);
-		return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+		return E_GW_CONNECTION_STATUS_NO_RESPONSE;
 	}
 
 	status = e_gw_connection_parse_response_status (response);
@@ -1188,7 +1234,7 @@ e_gw_connection_accept_request (EGwConnection *cnc, const char *id, const char *
 	response = e_gw_connection_send_message (cnc, msg);
         if (!response) {
                 g_object_unref (msg);
-                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+                return E_GW_CONNECTION_STATUS_NO_RESPONSE;
         }
 
         status = e_gw_connection_parse_response_status (response);
@@ -1215,7 +1261,7 @@ e_gw_connection_decline_request (EGwConnection *cnc, const char *id)
 	response = e_gw_connection_send_message (cnc, msg);
         if (!response) {
                 g_object_unref (msg);
-                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+                return E_GW_CONNECTION_STATUS_NO_RESPONSE;
         }
 
         status = e_gw_connection_parse_response_status (response);
@@ -1243,7 +1289,7 @@ e_gw_connection_retract_request (EGwConnection *cnc, const char *id, const char
 	response = e_gw_connection_send_message (cnc, msg);
         if (!response) {
                 g_object_unref (msg);
-                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+                return E_GW_CONNECTION_STATUS_NO_RESPONSE;
         }
 
         status = e_gw_connection_parse_response_status (response);
@@ -1281,6 +1327,14 @@ e_gw_connection_complete_request (EGwConnection *cnc, const char *id)
 	return status;
 }
 
+const char *
+e_gw_connection_get_version (EGwConnection *cnc)
+{
+	g_return_val_if_fail (E_IS_GW_CONNECTION (cnc), NULL);
+
+	return (const char *) cnc->priv->version;
+}
+
 const char *
 e_gw_connection_get_uri (EGwConnection *cnc)
 {
@@ -1324,6 +1378,14 @@ e_gw_connection_get_user_uuid (EGwConnection *cnc)
 	return (const char *) cnc->priv->user_uuid;
 }
 
+const char * 
+e_gw_connection_get_server_time (EGwConnection *cnc)
+{
+	g_return_val_if_fail (E_IS_GW_CONNECTION (cnc), NULL) ;
+
+	return (const char *) cnc->priv->server_time ;
+}
+
 static time_t
 timet_from_string (const char *str)
 {
@@ -1387,7 +1449,7 @@ e_gw_connection_get_date_from_string (const char *dtstring)
         int i, j, len = strlen (dtstring);
 	time_t t;
 	
-        str2 = g_malloc0 (len);
+        str2 = g_malloc0 (len+1);
         for (i = 0,j = 0; i < len; i++) {
                 if ((dtstring[i] != '-') && (dtstring[i] != ':')) {
 			str2[j] = dtstring[i];
@@ -1421,7 +1483,7 @@ e_gw_connection_create_book (EGwConnection *cnc, char *book_name, char**id)
 	response = e_gw_connection_send_message (cnc, msg);
         if (!response) {
                 g_object_unref (msg);
-                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+                return E_GW_CONNECTION_STATUS_NO_RESPONSE;
         }
 
         status = e_gw_connection_parse_response_status (response);
@@ -1449,16 +1511,28 @@ e_gw_connection_get_address_book_list (EGwConnection *cnc, GList **container_lis
 	SoupSoapMessage *msg;
 	SoupSoapResponse *response;
         EGwConnectionStatus status;
+	EGwConnectionPrivate *priv;
 	SoupSoapParameter *param;
 	SoupSoapParameter *type_param;
 	char *value;
-	
+	static GStaticMutex connecting = G_STATIC_MUTEX_INIT;
+
 	g_return_val_if_fail (E_IS_GW_CONNECTION (cnc), E_GW_CONNECTION_STATUS_UNKNOWN);
 	g_return_val_if_fail (container_list != NULL, E_GW_CONNECTION_STATUS_UNKNOWN);
 
+	priv = cnc->priv;
+	g_static_mutex_lock (&connecting);
+
+	if (priv->book_list) {
+		*container_list = priv->book_list;
+		g_static_mutex_unlock (&connecting);
+		return E_GW_CONNECTION_STATUS_OK;
+	}
+
 	msg = e_gw_message_new_with_header (cnc->priv->uri, cnc->priv->session_id, "getAddressBookListRequest");
         if (!msg) {
                 g_warning (G_STRLOC ": Could not build SOAP message");
+		g_static_mutex_unlock (&connecting);
                 return E_GW_CONNECTION_STATUS_UNKNOWN;
         }
 
@@ -1468,7 +1542,8 @@ e_gw_connection_get_address_book_list (EGwConnection *cnc, GList **container_lis
 	response = e_gw_connection_send_message (cnc, msg);
         if (!response) {
                 g_object_unref (msg);
-                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+		g_static_mutex_unlock (&connecting);
+                return E_GW_CONNECTION_STATUS_NO_RESPONSE;
         }
 
         status = e_gw_connection_parse_response_status (response);
@@ -1478,6 +1553,7 @@ e_gw_connection_get_address_book_list (EGwConnection *cnc, GList **container_lis
 		if (status == E_GW_CONNECTION_STATUS_INVALID_CONNECTION)
 			reauthenticate (cnc);
                 g_object_unref (response);
+		g_static_mutex_unlock (&connecting);
                 return status;
         }
 	
@@ -1485,6 +1561,7 @@ e_gw_connection_get_address_book_list (EGwConnection *cnc, GList **container_lis
 	param = soup_soap_response_get_first_parameter_by_name (response, "books");	
         if (!param) {
                 g_object_unref (response);
+		g_static_mutex_unlock (&connecting);
                 return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
         } else {
 		SoupSoapParameter *subparam;
@@ -1495,7 +1572,7 @@ e_gw_connection_get_address_book_list (EGwConnection *cnc, GList **container_lis
 				       
 			container = e_gw_container_new_from_soap_parameter (subparam);
 			if (container) {
-				*container_list = g_list_append (*container_list, container);
+				priv->book_list = g_list_append (priv->book_list, container);
 				type_param = soup_soap_parameter_get_first_child_by_name (subparam, "isPersonal");
 				value = NULL;
 				if (type_param)
@@ -1520,6 +1597,8 @@ e_gw_connection_get_address_book_list (EGwConnection *cnc, GList **container_lis
 	}
 
 	g_object_unref (response);
+	*container_list = priv->book_list;
+	g_static_mutex_unlock (&connecting);
 
         return status;
 }
@@ -1536,7 +1615,6 @@ e_gw_connection_get_address_book_id ( EGwConnection *cnc, char *book_name, char*
 
 	status = e_gw_connection_get_address_book_list (cnc, &container_list);
         if (status != E_GW_CONNECTION_STATUS_OK) {
-		e_gw_connection_free_container_list (container_list);
                 return status;
         }
 
@@ -1551,28 +1629,84 @@ e_gw_connection_get_address_book_id ( EGwConnection *cnc, char *book_name, char*
 		}
 	}
 
-	e_gw_connection_free_container_list (container_list);
-
 	return status;
 
 }
 
+EGwConnectionStatus
+e_gw_connection_modify_settings (EGwConnection *cnc, EGwSendOptions *opts)
+{
+	SoupSoapMessage *msg;
+	SoupSoapResponse *response;
+	EGwConnectionStatus status;
+	EGwConnectionPrivate *priv;
+
+	g_return_val_if_fail (E_IS_GW_CONNECTION (cnc), E_GW_CONNECTION_STATUS_INVALID_OBJECT);
+	g_return_val_if_fail (opts != NULL, E_GW_CONNECTION_STATUS_INVALID_OBJECT);
+
+	priv = cnc->priv;
+
+	msg = e_gw_message_new_with_header (cnc->priv->uri, cnc->priv->session_id, "modifySettingsRequest");
+	if (!msg) {
+                g_warning (G_STRLOC ": Could not build SOAP message");
+                return E_GW_CONNECTION_STATUS_UNKNOWN;
+        }
+	
+	if (!e_gw_sendoptions_form_message_to_modify (msg, opts, priv->opts)) {
+		g_warning (G_STRLOC ": Could not append changes to SOAP message");
+		g_object_unref (msg);
+		return E_GW_CONNECTION_STATUS_INVALID_OBJECT;
+	}
+
+       	e_gw_message_write_footer (msg);
+	
+        /* send message to server */
+        response = e_gw_connection_send_message (cnc, msg);
+        if (!response) {
+                g_object_unref (msg);
+                return E_GW_CONNECTION_STATUS_NO_RESPONSE;
+        }
+
+	status = e_gw_connection_parse_response_status (response);
+       	if (status != E_GW_CONNECTION_STATUS_OK) {
+		if (status == E_GW_CONNECTION_STATUS_INVALID_CONNECTION)
+			reauthenticate (cnc);
+		g_object_unref (response);
+                g_object_unref (msg);
+		return status;
+	} else {
+		g_object_unref (priv->opts);
+		priv->opts = NULL;
+		priv->opts = opts;
+	}
+
+	g_object_unref (response);
+        g_object_unref (msg);
+
+	return status;
+}
+
 EGwConnectionStatus
 e_gw_connection_get_settings (EGwConnection *cnc, EGwSendOptions **opts)
 {
 	SoupSoapMessage *msg;
 	SoupSoapResponse *response;
 	EGwConnectionStatus status;
-	SoupSoapParameter *param, *subparam;
+	SoupSoapParameter *param;
 	EGwConnectionPrivate *priv;
+	static GStaticMutex connecting = G_STATIC_MUTEX_INIT;	
+
+
+	g_return_val_if_fail (E_IS_GW_CONNECTION (cnc), E_GW_CONNECTION_STATUS_INVALID_OBJECT);
 
 	priv = cnc->priv;
-	
-	g_return_val_if_fail (E_IS_GW_CONNECTION (cnc), E_GW_CONNECTION_STATUS_INVALID_OBJECT);
+
+	g_static_mutex_lock (&connecting);
 
 	if (priv->opts) {
 		g_object_ref (priv->opts);
-		opts = &priv->opts;
+		*opts = priv->opts;
+		g_static_mutex_unlock (&connecting);
 		
 		return E_GW_CONNECTION_STATUS_OK;
 	}
@@ -1580,6 +1714,7 @@ e_gw_connection_get_settings (EGwConnection *cnc, EGwSendOptions **opts)
 	msg = e_gw_message_new_with_header (cnc->priv->uri, cnc->priv->session_id, "getSettingsRequest");
 	if (!msg) {
                 g_warning (G_STRLOC ": Could not build SOAP message");
+		g_static_mutex_unlock (&connecting);
                 return E_GW_CONNECTION_STATUS_UNKNOWN;
         }
        	e_gw_message_write_footer (msg);
@@ -1588,7 +1723,8 @@ e_gw_connection_get_settings (EGwConnection *cnc, EGwSendOptions **opts)
         response = e_gw_connection_send_message (cnc, msg);
         if (!response) {
                 g_object_unref (msg);
-                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+		g_static_mutex_unlock (&connecting);
+                return E_GW_CONNECTION_STATUS_NO_RESPONSE;
         }
 
         status = e_gw_connection_parse_response_status (response);
@@ -1597,6 +1733,7 @@ e_gw_connection_get_settings (EGwConnection *cnc, EGwSendOptions **opts)
 			reauthenticate (cnc);
 		g_object_unref (response);
                 g_object_unref (msg);
+		g_static_mutex_unlock (&connecting);
 		return status;
 	}
 	
@@ -1604,28 +1741,47 @@ e_gw_connection_get_settings (EGwConnection *cnc, EGwSendOptions **opts)
         if (!param) {
                 g_object_unref (response);
                 g_object_unref (msg);
+		g_static_mutex_unlock (&connecting);
                 return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
         } else 
-		*opts =	e_gw_sendoptions_new_from_soap_parameter (param);
-		
+		priv->opts = e_gw_sendoptions_new_from_soap_parameter (param);
+
+	g_object_ref (priv->opts);
+	*opts = priv->opts;
+	g_object_unref (response);
+        g_object_unref (msg);
+	g_static_mutex_unlock (&connecting);
 	
 	return E_GW_CONNECTION_STATUS_OK;
 }
 
 EGwConnectionStatus 
-e_gw_connection_get_categories (EGwConnection *cnc, GHashTable *categories_by_id, GHashTable *categories_by_name)
+e_gw_connection_get_categories (EGwConnection *cnc, GHashTable **categories_by_id, GHashTable **categories_by_name)
 {
 	SoupSoapMessage *msg;
         SoupSoapResponse *response;
         EGwConnectionStatus status;
+	EGwConnectionPrivate *priv;
         SoupSoapParameter *param, *subparam, *second_level_child;
 	char *id, *name;
         g_return_val_if_fail (E_IS_GW_CONNECTION (cnc), E_GW_CONNECTION_STATUS_INVALID_OBJECT);
-	
+	static GStaticMutex connecting = G_STATIC_MUTEX_INIT;
+
+	priv = cnc->priv;
+	g_static_mutex_lock (&connecting);
+
+	if (priv->categories_by_id && priv->categories_by_name) {
+		*categories_by_id = priv->categories_by_id;
+		*categories_by_name = priv->categories_by_name;
+		g_static_mutex_unlock (&connecting);
+		return E_GW_CONNECTION_STATUS_OK;
+	}
+
 	/* build the SOAP message */
         msg = e_gw_message_new_with_header (cnc->priv->uri, cnc->priv->session_id, "getCategoryListRequest");
         if (!msg) {
                 g_warning (G_STRLOC ": Could not build SOAP message");
+		g_static_mutex_unlock (&connecting);
                 return E_GW_CONNECTION_STATUS_UNKNOWN;
         }
        	e_gw_message_write_footer (msg);
@@ -1634,7 +1790,8 @@ e_gw_connection_get_categories (EGwConnection *cnc, GHashTable *categories_by_id
         response = e_gw_connection_send_message (cnc, msg);
         if (!response) {
                 g_object_unref (msg);
-                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+		g_static_mutex_unlock (&connecting);
+                return E_GW_CONNECTION_STATUS_NO_RESPONSE;
         }
 
         status = e_gw_connection_parse_response_status (response);
@@ -1643,6 +1800,7 @@ e_gw_connection_get_categories (EGwConnection *cnc, GHashTable *categories_by_id
 			reauthenticate (cnc);
 		g_object_unref (response);
                 g_object_unref (msg);
+		g_static_mutex_unlock (&connecting);
 		return status;
 	}
 
@@ -1651,9 +1809,13 @@ e_gw_connection_get_categories (EGwConnection *cnc, GHashTable *categories_by_id
         if (!param) {
                 g_object_unref (response);
                 g_object_unref (msg);
+		g_static_mutex_unlock (&connecting);
                 return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
         }
 	
+	priv->categories_by_id = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
+	priv->categories_by_name = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
+
 	for (subparam = soup_soap_parameter_get_first_child_by_name (param, "category");
 	     subparam != NULL;
 	     subparam = soup_soap_parameter_get_next_child_by_name (subparam, "category")) {
@@ -1669,9 +1831,9 @@ e_gw_connection_get_categories (EGwConnection *cnc, GHashTable *categories_by_id
 			g_free (id);
 			id = components[0];
 			if (categories_by_id) 
-				g_hash_table_insert (categories_by_id, g_strdup (id), g_strdup (name));
+				g_hash_table_insert (priv->categories_by_id, g_strdup (id), g_strdup (name));
 			if (categories_by_name) 
-				g_hash_table_insert (categories_by_name, g_strdup (name), g_strdup (id));
+				g_hash_table_insert (priv->categories_by_name, g_strdup (name), g_strdup (id));
 			g_strfreev (components);
 			g_free (name);
 		}
@@ -1681,10 +1843,11 @@ e_gw_connection_get_categories (EGwConnection *cnc, GHashTable *categories_by_id
 	/* free memory */
         g_object_unref (response);
 	g_object_unref (msg);
+	*categories_by_id = priv->categories_by_id;
+	*categories_by_name = priv->categories_by_name;
+	g_static_mutex_unlock (&connecting);
 
         return E_GW_CONNECTION_STATUS_OK;
-
-
 }
 
 EGwConnectionStatus 
@@ -1717,7 +1880,7 @@ e_gw_connection_add_members (EGwConnection *cnc, const char *group_id, GList *me
 	response = e_gw_connection_send_message (cnc, msg);
         if (!response) {
                 g_object_unref (msg);
-                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+                return E_GW_CONNECTION_STATUS_NO_RESPONSE;
         }
 
         status = e_gw_connection_parse_response_status (response);
@@ -1761,7 +1924,7 @@ e_gw_connection_remove_members (EGwConnection *cnc, const char *group_id, GList
 	response = e_gw_connection_send_message (cnc, msg);
         if (!response) {
                 g_object_unref (msg);
-                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+                return E_GW_CONNECTION_STATUS_NO_RESPONSE;
         }
 
         status = e_gw_connection_parse_response_status (response);
@@ -1800,7 +1963,7 @@ e_gw_connection_create_cursor (EGwConnection *cnc, const char *container, const
 	response = e_gw_connection_send_message (cnc, msg);
         if (!response) {
                 g_object_unref (msg);
-                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+                return E_GW_CONNECTION_STATUS_NO_RESPONSE;
         }
 	
 	status = e_gw_connection_parse_response_status (response);
@@ -1852,7 +2015,7 @@ e_gw_connection_destroy_cursor (EGwConnection *cnc, const char *container,  int
 	response = e_gw_connection_send_message (cnc, msg);
         if (!response) {
                 g_object_unref (msg);
-                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+                return E_GW_CONNECTION_STATUS_NO_RESPONSE;
         }
 	
 	status = e_gw_connection_parse_response_status (response);
@@ -1877,8 +2040,8 @@ e_gw_connection_position_cursor (EGwConnection *cnc, const char *container, int
 	g_return_val_if_fail ((container != NULL), E_GW_CONNECTION_STATUS_UNKNOWN);
 
 	msg = e_gw_message_new_with_header (cnc->priv->uri, cnc->priv->session_id, "positionCursorRequest");
-	e_gw_message_write_string_parameter (msg, "container", NULL, container);
 	e_gw_message_write_int_parameter (msg, "cursor", NULL, cursor);
+	e_gw_message_write_string_parameter (msg, "container", NULL, container);
 	e_gw_message_write_string_parameter (msg, "seek", NULL, seek);
 	e_gw_message_write_int_parameter (msg, "offset", NULL, offset);
 	
@@ -1887,7 +2050,7 @@ e_gw_connection_position_cursor (EGwConnection *cnc, const char *container, int
 	response = e_gw_connection_send_message (cnc, msg);
         if (!response) {
                 g_object_unref (msg);
-                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+                return E_GW_CONNECTION_STATUS_NO_RESPONSE;
         }
 	
 	status = e_gw_connection_parse_response_status (response);
@@ -1897,7 +2060,7 @@ e_gw_connection_position_cursor (EGwConnection *cnc, const char *container, int
 }
 
 EGwConnectionStatus
-e_gw_connection_read_cursor (EGwConnection *cnc, const char *container, int cursor, gboolean forward, int count, GList **item_list)
+e_gw_connection_read_cursor (EGwConnection *cnc, const char *container, int cursor, gboolean forward, int count, const char *cursor_seek, GList **item_list)
 {
 	SoupSoapMessage *msg;
 	SoupSoapResponse *response;
@@ -1908,11 +2071,12 @@ e_gw_connection_read_cursor (EGwConnection *cnc, const char *container, int curs
 	g_return_val_if_fail ((container != NULL), E_GW_CONNECTION_STATUS_UNKNOWN);
 	
 	msg = e_gw_message_new_with_header (cnc->priv->uri, cnc->priv->session_id, "readCursorRequest");
-	e_gw_message_write_string_parameter (msg, "container", NULL, container);
 	e_gw_message_write_int_parameter (msg, "cursor", NULL, cursor);
 	/* there is problem in read curosr if you set this, uncomment after the problem 
 	   is fixed in server */
 	e_gw_message_write_string_parameter (msg, "forward", NULL, forward ? "true": "false");
+	e_gw_message_write_string_parameter (msg, "position", NULL, cursor_seek);
+	e_gw_message_write_string_parameter (msg, "container", NULL, container);
 	e_gw_message_write_int_parameter (msg, "count", NULL, count);
 	
 	e_gw_message_write_footer (msg);
@@ -1920,7 +2084,7 @@ e_gw_connection_read_cursor (EGwConnection *cnc, const char *container, int curs
 	response = e_gw_connection_send_message (cnc, msg);
         if (!response) {
                 g_object_unref (msg);
-                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+                return E_GW_CONNECTION_STATUS_NO_RESPONSE;
         }
 	
 	status = e_gw_connection_parse_response_status (response);
@@ -1956,7 +2120,7 @@ e_gw_connection_read_cursor (EGwConnection *cnc, const char *container, int curs
         return E_GW_CONNECTION_STATUS_OK;
 }
 
-EGwConnectionStatus e_gw_connection_get_quick_messages (EGwConnection *cnc, const char *container, const char *view, const char *start_date, const char *message_list, const char *item_types, const char *item_sources, int count, GSList **item_list)
+EGwConnectionStatus e_gw_connection_get_quick_messages (EGwConnection *cnc, const char *container, const char *view, char **start_date, const char *message_list, const char *item_types, const char *item_sources, int count, GSList **item_list)
 {
 	SoupSoapMessage *msg;
 	SoupSoapResponse *response;
@@ -1969,7 +2133,7 @@ EGwConnectionStatus e_gw_connection_get_quick_messages (EGwConnection *cnc, cons
 	msg = e_gw_message_new_with_header (cnc->priv->uri, cnc->priv->session_id, "getQuickMessagesRequest");
 	e_gw_message_write_string_parameter (msg, "list", NULL, message_list);
 	if (start_date)
-		e_gw_message_write_string_parameter (msg, "startDate", NULL, start_date);
+		e_gw_message_write_string_parameter (msg, "startDate", NULL, *start_date);
 	if (container)
 		e_gw_message_write_string_parameter (msg, "container", NULL, container);
 	if (item_types) 
@@ -1986,7 +2150,7 @@ EGwConnectionStatus e_gw_connection_get_quick_messages (EGwConnection *cnc, cons
 	response = e_gw_connection_send_message (cnc, msg);
         if (!response) {
                 g_object_unref (msg);
-                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+                return E_GW_CONNECTION_STATUS_NO_RESPONSE;
         }
 	
 	status = e_gw_connection_parse_response_status (response);
@@ -2007,6 +2171,20 @@ EGwConnectionStatus e_gw_connection_get_quick_messages (EGwConnection *cnc, cons
                 return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
         }
 
+	if (start_date && *start_date && !strcmp (message_list, "New")) {
+		subparam = soup_soap_response_get_first_parameter_by_name (response, "startDate");
+		if (subparam) {
+			char *date;
+
+			date = soup_soap_parameter_get_string_value (subparam);
+			if (date)
+				g_free (*start_date), *start_date = date;
+			else 
+				return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+		} else 
+			return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+	}
+	
 	if (!strcmp (message_list, "All")) { 
 		/* We are  interested only in getting the ids */
 		for (subparam = soup_soap_parameter_get_first_child_by_name (param, "item");
@@ -2075,7 +2253,7 @@ e_gw_connection_create_folder(EGwConnection *cnc, const char *parent_name,const
 	response =  e_gw_connection_send_message (cnc, msg);
         if (!response) {
                 g_object_unref (msg);
-                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+                return E_GW_CONNECTION_STATUS_NO_RESPONSE;
         }
 	status = e_gw_connection_parse_response_status (response);
         if (status != E_GW_CONNECTION_STATUS_OK) {
@@ -2127,7 +2305,7 @@ e_gw_connection_get_attachment (EGwConnection *cnc, const char *id, int offset,
         response = e_gw_connection_send_message (cnc, msg);
         if (!response) {
                 g_object_unref (msg);
-                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+                return E_GW_CONNECTION_STATUS_NO_RESPONSE;
         }
 
         status = e_gw_connection_parse_response_status (response);
@@ -2181,7 +2359,7 @@ e_gw_connection_add_item (EGwConnection *cnc, const char *container, const char
 	response = e_gw_connection_send_message (cnc, msg);
 	if (!response) {
 		g_object_unref (msg);
-		return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+		return E_GW_CONNECTION_STATUS_NO_RESPONSE;
 	}
 
 	status = e_gw_connection_parse_response_status (response);
@@ -2221,7 +2399,7 @@ e_gw_connection_add_items (EGwConnection *cnc, const char *container, GList *ite
 	response = e_gw_connection_send_message (cnc, msg);
 	if (!response) {
 		g_object_unref (msg);
-		return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+		return E_GW_CONNECTION_STATUS_NO_RESPONSE;
 	}
 
 	status = e_gw_connection_parse_response_status (response);
@@ -2264,7 +2442,7 @@ e_gw_connection_rename_folder (EGwConnection *cnc, const char *id ,const char *n
 	response = e_gw_connection_send_message (cnc, msg);
 	if (!response) {
 		g_object_unref (msg);
-		return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+		return E_GW_CONNECTION_STATUS_NO_RESPONSE;
 	}
 
 	status = e_gw_connection_parse_response_status (response);
@@ -2329,7 +2507,7 @@ e_gw_connection_move_item (EGwConnection *cnc, const char *id, const char *dest_
 	response = e_gw_connection_send_message (cnc, msg);
 	if (!response) {
 		g_object_unref (msg);
-		return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+		return E_GW_CONNECTION_STATUS_NO_RESPONSE;
 	}
 
 	status = e_gw_connection_parse_response_status (response);
@@ -2365,7 +2543,7 @@ e_gw_connection_accept_shared_folder (EGwConnection *cnc, gchar *name, gchar *co
 	response =  e_gw_connection_send_message (cnc, msg);
 	if (!response) {
 		g_object_unref (msg);
-		return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+		return E_GW_CONNECTION_STATUS_NO_RESPONSE;
 	}
 	status = e_gw_connection_parse_response_status (response);
 	if (status == E_GW_CONNECTION_STATUS_INVALID_CONNECTION)
@@ -2391,7 +2569,7 @@ e_gw_connection_purge_deleted_items (EGwConnection *cnc)
 	response =  e_gw_connection_send_message (cnc, msg);
 	if (!response) {
 		g_object_unref (msg);
-		return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+		return E_GW_CONNECTION_STATUS_NO_RESPONSE;
 	}
 	status = e_gw_connection_parse_response_status (response);
 	if (status == E_GW_CONNECTION_STATUS_INVALID_CONNECTION)
@@ -2426,7 +2604,7 @@ e_gw_connection_mark_read(EGwConnection *cnc, GList *item_ids)
 	response = e_gw_connection_send_message (cnc, msg);
 	if (!response) {
 		g_object_unref (msg);
-		return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+		return E_GW_CONNECTION_STATUS_NO_RESPONSE;
 	}
 
 	status = e_gw_connection_parse_response_status (response);
@@ -2461,7 +2639,7 @@ e_gw_connection_mark_unread(EGwConnection *cnc, GList *item_ids)
 	response = e_gw_connection_send_message (cnc, msg);
 	if (!response) {
 		g_object_unref (msg);
-		return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+		return E_GW_CONNECTION_STATUS_NO_RESPONSE;
 	}
 
 	status = e_gw_connection_parse_response_status (response);
@@ -2473,3 +2651,60 @@ e_gw_connection_mark_unread(EGwConnection *cnc, GList *item_ids)
 
 	return status;
 }
+
+EGwConnectionStatus
+e_gw_connection_reply_item (EGwConnection *cnc, const char *id, const char *view, EGwItem **item)
+{
+	SoupSoapMessage *msg;
+        SoupSoapResponse *response;
+        EGwConnectionStatus status;
+        SoupSoapParameter *param;
+        
+	g_return_val_if_fail (E_IS_GW_CONNECTION (cnc), E_GW_CONNECTION_STATUS_INVALID_OBJECT);
+	
+	/* build the SOAP message */
+        msg = e_gw_message_new_with_header (cnc->priv->uri, cnc->priv->session_id, "replyRequest");
+        if (!msg) {
+                g_warning (G_STRLOC ": Could not build SOAP message");
+                return E_GW_CONNECTION_STATUS_UNKNOWN;
+        }
+	
+	e_gw_message_write_string_parameter (msg, "id", NULL, id);
+
+	if (view)
+		e_gw_message_write_string_parameter (msg, "view", NULL, view) ;
+	e_gw_message_write_footer (msg);
+        
+	/* send message to server */
+        response = e_gw_connection_send_message (cnc, msg);
+        if (!response) {
+                g_object_unref (msg);
+                return E_GW_CONNECTION_STATUS_NO_RESPONSE;
+        }
+        
+	status = e_gw_connection_parse_response_status (response);
+        if (status != E_GW_CONNECTION_STATUS_OK) {
+		if (status == E_GW_CONNECTION_STATUS_INVALID_CONNECTION)
+			reauthenticate (cnc);
+		g_object_unref (response);
+                g_object_unref (msg);
+		return status;
+	}
+
+	/* if status is OK - parse result. return the list */	
+	param = soup_soap_response_get_first_parameter_by_name (response, "item");
+        if (!param) {
+                g_object_unref (response);
+                g_object_unref (msg);
+                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+        }
+       	
+	*item = e_gw_item_new_from_soap_parameter (cnc->priv->user_email, "", param);
+	
+               
+	/* free memory */
+        g_object_unref (response);
+	g_object_unref (msg);
+
+        return E_GW_CONNECTION_STATUS_OK;
+}
